"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Lobby = void 0;
const mydog_1 = require("mydog");
const group_1 = require("./group");
const match_1 = require("./match");
const player_1 = require("./player");
const room_1 = require("./room");
/**
 * 大厅类
 */
class Lobby {
    constructor() {
        this.roomId = 0; // 房间id
        this.groupId = 0; // 队组id
        this.matchId = 0; // 匹配id
        this.playerId = 0; // 玩家id
        this.roomList = {}; // 房间集合
        this.groupList = {}; // 队组集合
        this.matchList = {}; // 匹配集合
        this.playerList = {}; // 玩家集合
        console.log("大厅启动");
        setTimeout(this.update.bind(this), 1000);
    }
    /**
     * 登录
     * @param name 昵称
     */
    login(name, next) {
        let id = this.getPlayerId();
        this.createPlayer(id, name, false);
        next({ type: 0, data: [id, name] });
        return id;
    }
    /**
     * 队组
     * @param msg 数据
     */
    group(msg) {
        let type = msg.type;
        let uid = msg.uid;
        let groupId = msg.groupId;
        let removeId = msg.removeId;
        // 创建队组
        if (type == 0) {
            this.createGroup(uid);
        }
        // 加入队组
        if (type == 1) {
            this.joinGroup(uid, groupId);
        }
        // 离开队组
        if (type == 2) {
            this.leaveGroup(uid, groupId);
        }
        // 队组中移除指定玩家
        if (type == 3) {
            // console.log("队组中移除指定玩家");
            // console.log(msg);
            this.removeGroup(uid, groupId, removeId);
        }
    }
    /**
     * 匹配
     * @param msg 数据
     */
    match(msg) {
        let type = msg.type;
        let uid = msg.uid;
        let groupId = msg.groupId;
        // 开始匹配
        if (type == 0) {
            let group = this.getGroup(groupId);
            if (group != undefined) {
                // 不在匹配中
                if (!group.isMatch) {
                    if (group.playerList[0].id == uid) {
                        group.isMatch = true;
                        console.log("开始匹配");
                        // 匹配列队是否为空
                        if (Object.keys(this.matchList).length == 0) {
                            // 创建一个匹配队列
                            this.createMatch(group.playerList);
                            console.log("创建一个匹配队列");
                        }
                        else {
                            // 尝试加入一个匹配队列
                            for (let id in this.matchList) {
                                let match = this.matchList[id];
                                let matchCount = match.playerList.length + group.playerList.length;
                                if (matchCount <= match.matchCount) {
                                    match.addGroup(group.playerList);
                                    let uids = [];
                                    for (let i = 0; i < group.playerList.length; i++) {
                                        let player = group.playerList[i];
                                        uids.push(player.id);
                                    }
                                    mydog_1.app.sendMsgByUid(3 /* connector_main_match */, { type: 0, data: 0 }, uids);
                                    console.log("加入匹配队列");
                                    return;
                                }
                            }
                            // 没有符合条件的匹配队列，创建匹配
                            this.createMatch(group.playerList);
                            console.log("没有符合条件的匹配队列，创建匹配");
                        }
                    }
                    else {
                        this.sendErrorCode(2, uid);
                    }
                }
            }
        }
        // 取消匹配
        if (type == 1) {
            let group = this.getGroup(groupId);
            if (group != undefined) {
                let player = this.getPlayer(uid);
                let index = group.playerList.indexOf(player);
                if (index != -1) {
                    console.log("取消匹配");
                    let match = this.getMatch(player.matchId);
                    match.removeGroup(group.playerList);
                    let uids = [];
                    for (let i = 0; i < group.playerList.length; i++) {
                        let player = group.playerList[i];
                        uids.push(player.id);
                    }
                    mydog_1.app.sendMsgByUid(3 /* connector_main_match */, { type: 1, data: 0 }, uids);
                }
            }
        }
    }
    /**
     * 帧数据
     * @param msg 数据
     * @returns 无
     */
    frame(msg) {
        let roomId = msg.roomId;
        let data = msg.data;
        // console.log(msg);
        let room = this.getRoom(roomId);
        if (room == undefined) {
            return;
        }
        // 新增帧数据
        room.addFrame(data);
    }
    /**
     * 更新客户端队组数据
     * @param groupId 队组id
     * @returns 无
     */
    updateGroup(groupId) {
        let group = this.getGroup(groupId);
        if (group == undefined) {
            return;
        }
        let uids = [];
        let players = group.playerList;
        for (let i = 0; i < players.length; i++) {
            uids.push(players[i].id);
        }
        mydog_1.app.sendMsgByUid(2 /* connector_main_group */, { type: 4, data: [group.id, group.playerList] }, uids);
    }
    /**
     * 创建房间
     * @param players 玩家列表
     * @return 房间id
     */
    createRoom(players) {
        let id = this.getRoomId();
        let room = new room_1.Room(id, players);
        this.roomList[id] = room;
        return id;
    }
    /**
     * 创建队组
     * @param uid 玩家id
     */
    createGroup(uid) {
        let player = this.getPlayer(uid);
        if (player.groupId == -1) {
            let id = this.getGroupId();
            let group = new group_1.Group(id);
            group.addPlayer(player);
            this.groupList[id] = group;
            mydog_1.app.sendMsgByUid(2 /* connector_main_group */, { type: 0, data: 0 }, [player.id]);
            this.updateGroup(group.id);
        }
    }
    /**
     * 加入队组
     * @param uid 玩家id
     * @param groupId 队组id
     */
    joinGroup(uid, groupId) {
        let player = this.getPlayer(uid);
        let group = this.getGroup(groupId);
        if (group == undefined) {
            this.sendErrorCode(0, uid);
            return;
        }
        if (!group.isJoin()) {
            this.sendErrorCode(0, uid);
            return;
        }
        // if (group.playerList[0].isRobot) {
        //     console.log("队伍不存在!");
        //     return;
        // }
        group.addPlayer(player);
        mydog_1.app.sendMsgByUid(2 /* connector_main_group */, { type: 1, data: 0 }, [player.id]);
        this.updateGroup(group.id);
    }
    /**
     * 离开队组
     * @param uid 玩家id
     * @param groupId 队组id
     */
    leaveGroup(uid, groupId) {
        let player = this.getPlayer(uid);
        let group = this.getGroup(groupId);
        if (group == undefined) {
            return;
        }
        if (!group.isLeave()) {
            this.sendErrorCode(1, uid);
            return;
        }
        group.removePlayer(player);
        mydog_1.app.sendMsgByUid(2 /* connector_main_group */, { type: 2, data: 0 }, [player.id]);
        this.updateGroup(group.id);
    }
    /**
     * 移除队组中的指定玩家
     * @param uid 队长 玩家id
     * @param groupId 队组id
     * @param removeId 被移除玩家在队组中的索引
     */
    removeGroup(uid, groupId, removeId) {
        let player = this.getPlayer(uid);
        let group = this.getGroup(groupId);
        if (group == undefined) {
            return;
        }
        // 不是队长?
        if (group.playerList[0] != player) {
            // console.log("不是队长");
            this.sendErrorCode(2, uid);
            return;
        }
        // 不可以离开?
        if (!group.isLeave()) {
            this.sendErrorCode(1, uid);
            return;
        }
        let removePlayer = group.playerList[removeId];
        // 玩家不存在?
        if (removePlayer == undefined) {
            return;
        }
        group.removePlayer(removePlayer);
        mydog_1.app.sendMsgByUid(2 /* connector_main_group */, { type: 3, data: 0 }, [removePlayer.id]);
        this.updateGroup(group.id);
    }
    /**
     * 创建匹配
     * @param id 匹配id
     * @param players 玩家列表
     */
    createMatch(players) {
        let id = this.getMatchId();
        let match = new match_1.Match(id);
        match.addGroup(players);
        this.matchList[id] = match;
        let uids = [];
        for (let i = 0; i < players.length; i++) {
            let player = players[i];
            uids.push(player.id);
        }
        mydog_1.app.sendMsgByUid(3 /* connector_main_match */, { type: 0, data: 0 }, uids);
    }
    /**
     * 创建玩家
     * @param id id
     * @param name 昵称
     */
    createPlayer(id, name, isRobot) {
        let player = new player_1.Player(id, name, isRobot);
        this.playerList[id] = player;
    }
    /**
     * 获取房间唯一id
     * @returns 唯一id
     */
    getRoomId() {
        this.roomId++;
        let id = this.roomId;
        return id;
    }
    /**
     * 获取队组唯一id
     * @returns 唯一id
     */
    getGroupId() {
        this.groupId++;
        let id = this.groupId;
        return id;
    }
    /**
     * 获取匹配唯一id
     * @returns 唯一id
     */
    getMatchId() {
        this.matchId++;
        let id = this.matchId;
        return id;
    }
    /**
     * 获取玩家唯一id
     * @returns 唯一id
     */
    getPlayerId() {
        this.playerId++;
        let id = this.playerId;
        return id;
    }
    /**
     * 获取房间对象
     * @param id 房间id
     * @returns 房间对象
     */
    getRoom(id) {
        return this.roomList[id];
    }
    /**
     * 获取队组对象
     * @param id 队组id
     * @returns 队组对象
     */
    getGroup(id) {
        return this.groupList[id];
    }
    /**
     * 获取匹配对象
     * @param id 匹配id
     * @returns 匹配对象
     */
    getMatch(id) {
        return this.matchList[id];
    }
    /**
     * 获取玩家对象
     * @param id 玩家id
     * @returns 玩家对象
     */
    getPlayer(id) {
        return this.playerList[id];
    }
    sendErrorCode(type, uid) {
        mydog_1.app.sendMsgByUid(6 /* connector_main_error */, { type: type }, [uid]);
    }
    /**
     * 更新循环
     */
    update() {
        setTimeout(this.update.bind(this), 1000);
        for (let i = 0; i < 6; i++) {
            this.updateAIMatch();
        }
        this.updateMatch();
        this.updateHeartbeat();
        // console.log("玩家数量: " + Object.keys(this.playerList).length + ", " + "队组数量: " + Object.keys(this.groupList).length + ", " + "匹配数量: " + Object.keys(this.matchList).length + ", " + "房间数量: " + Object.keys(this.roomList).length);
    }
    /**
     * 更新匹配循环
     */
    updateMatch() {
        let removeList = [];
        for (let id in this.matchList) {
            let match = this.matchList[id];
            if (match.isMatch()) {
                // console.log("匹配成功, matchId: " + match.id);
                let uids = [];
                for (let i = 0; i < match.playerList.length; i++) {
                    let player = match.playerList[i];
                    player.gId = player.groupId;
                    if (!player.isRobot) {
                        uids.push(player.id);
                    }
                }
                let roomId = this.createRoom(match.playerList);
                mydog_1.app.sendMsgByUid(3 /* connector_main_match */, { type: 2, data: { roomId: roomId } }, uids);
                removeList.push(match.id);
            }
        }
        // 移除匹配成功的匹配列队 以及 队组列队
        for (let i = 0; i < removeList.length; i++) {
            let id = removeList[i];
            let match = this.matchList[id];
            let groupIds = [];
            for (let j = 0; j < match.playerList.length; j++) {
                let groupId = match.playerList[j].groupId;
                if (groupIds.indexOf(groupId) == -1) {
                    groupIds.push(groupId);
                }
            }
            for (let j = 0; j < groupIds.length; j++) {
                let id = groupIds[j];
                delete this.groupList[id];
            }
            delete this.matchList[id];
        }
    }
    /**
     * 更新机器人匹配循环
     */
    updateAIMatch() {
        for (let id in this.matchList) {
            let match = this.matchList[id];
            if (!match.isMatch()) {
                let id = this.getPlayerId();
                this.createPlayer(id, "ai_" + id, true);
                this.createGroup(id);
                let player = this.getPlayer(id);
                match.addGroup([player]);
            }
        }
    }
    /**
     * 更新玩家心跳循环
     */
    updateHeartbeat() {
        let playerIds = [];
        for (let id in this.playerList) {
            let player = this.playerList[id];
            if (!player.isRobot && player.connectState) {
                player.heartbeat--;
                // console.log(player.name + "/heartbeat: " + player.heartbeat);
                // 心跳为0
                if (player.heartbeat == 0) {
                    if (player.roomId != -1) {
                        player.connectState = false;
                    }
                    playerIds.push(player.id);
                }
            }
        }
        // 移除
        for (let i = 0; i < playerIds.length; i++) {
            // 关闭连接
            let id = playerIds[i];
            let session = mydog_1.app.getSession(id);
            if (session != null) {
                session.close();
            }
            // 移除队组的玩家
            let player = this.getPlayer(id);
            let groupId = player.groupId;
            let group = this.getGroup(player.groupId);
            if (group != undefined) {
                group.removePlayer(player);
                this.updateGroup(groupId);
            }
            // 移除玩家
            if (player.connectState) {
                delete this.playerList[id];
            }
        }
    }
}
exports.Lobby = Lobby;
