"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Room = void 0;
const mydog_1 = require("mydog");
/**
 * 房间类
 */
class Room {
    constructor(id, players) {
        /**
         * 房间id
         */
        this.id = -1;
        /**
         * 房间玩家列表
         */
        this.playerList = [];
        this.seed = 999; // 随机数种子
        this.frameList = []; // 帧数据列表
        this.frameIndex = 0; // 帧索引
        this.frameRate = 1000 / 20; // 帧速率
        this.gameTime = (20 * (20 * 60)) + 5; // 游戏时间
        this.id = id;
        this.playerList = players;
        this.init();
    }
    // 初始化
    init() {
        let gIds = [];
        for (let i = 0; i < this.playerList.length; i++) {
            let player = this.playerList[i];
            // console.log(player.gId);
            if (gIds.indexOf(player.gId) == -1) {
                gIds.push(player.gId);
            }
        }
        for (let i = 0; i < this.playerList.length; i++) {
            let player = this.playerList[i];
            for (let j = 0; j < gIds.length; j++) {
                if (player.gId == gIds[j]) {
                    player.teamId = j;
                }
            }
        }
        // console.log(gIds);
        this.seed = Math.floor(Math.random() * 100000000);
        this.frameList.push([]);
        let frame = { c: 0, d: [this.seed] };
        this.frameList[this.frameIndex].push(frame);
        for (let i = 0; i < this.playerList.length; i++) {
            let player = this.playerList[i];
            player.roomId = this.id;
            let frame = { c: 1, d: [player.id, player.name, player.isRobot, player.teamId] };
            this.frameList[this.frameIndex].push(frame);
        }
        setTimeout(this.update.bind(this), 1000);
    }
    // 更新房间循环
    update() {
        if (this.gameTime == 0) {
            return;
        }
        setTimeout(this.update.bind(this), this.frameRate);
        this.sendFrame();
        this.updateTime();
    }
    /**
     * 加入帧数据
     * @param frameData 帧数据包
     */
    addFrame(frameData) {
        for (let i = 0; i < frameData.length; i++) {
            this.frameList[this.frameIndex].push(frameData[i]);
        }
    }
    /**
     * 退出房间
     * @param uid 玩家uid
     */
    quitRoom(uid) {
        let player = this.getPlayer(uid);
        let index = this.playerList.indexOf(player);
        if (index != -1) {
            // console.log("玩家" + uid + "退出房间");
            player.init();
            this.playerList.splice(index, 1);
        }
    }
    /**
     * 断线重连
     * @param uid 玩家uid
     * @param frameIndex 玩家帧索引
     */
    reconnect(uid, frameIndex) {
        let player = this.getPlayer(uid);
        player.connectState = true;
        player.frameIndex = frameIndex;
        player.isReconnect = true;
    }
    /**
     * 发送帧数据
     */
    sendFrame() {
        let uidList = [];
        let reconnectList = [];
        for (let i = 0; i < this.playerList.length; i++) {
            let player = this.playerList[i];
            if (!player.isRobot) {
                if (!player.isReconnect) {
                    uidList.push(player.id);
                }
                else {
                    reconnectList.push(player.id);
                }
            }
        }
        if (reconnectList.length > 0) {
            // console.log(reconnectList);
            for (let i = 0; i < reconnectList.length; i++) {
                let frameData = [];
                let player = this.getPlayer(reconnectList[i]);
                // console.log(player);
                for (let j = player.frameIndex; j < this.frameList.length; j++) {
                    frameData.push(this.frameList[j]);
                }
                mydog_1.app.sendMsgByUid(5 /* connector_main_room */, { type: 1, data: frameData }, [player.id]);
                player.isReconnect = false;
            }
        }
        if (uidList.length > 0) {
            let frameData = this.frameList[this.frameIndex];
            mydog_1.app.sendMsgByUid(4 /* connector_main_frame */, { type: 0, data: frameData }, uidList);
        }
        this.frameList.push([]);
        this.frameIndex++;
    }
    /**
     * 更新房间时间
     */
    updateTime() {
        this.gameTime -= 1;
        if (this.gameTime == 0) {
            console.log("游戏结束");
            let lobby = mydog_1.app.get("Lobby");
            // 移除机器人
            let uids = [];
            for (let i = 0; i < this.playerList.length; i++) {
                let player = this.playerList[i];
                if (player.isRobot) {
                    uids.push(player.id);
                }
                else {
                    player.init();
                }
            }
            for (let i = 0; i < uids.length; i++) {
                let uid = uids[i];
                delete lobby.playerList[uid];
            }
            // 房间销毁
            delete lobby.roomList[this.id];
        }
    }
    /**
     * 获取玩家对象
     * @param uid 玩家uid
     * @returns 玩家对象
     */
    getPlayer(uid) {
        let index = -1;
        for (let i = 0; i < this.playerList.length; i++) {
            let player = this.playerList[i];
            if (player.id == uid) {
                index = i;
                break;
            }
        }
        return this.playerList[index];
    }
}
exports.Room = Room;
