"use strict";
/**
 * app class
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const appUtil = __importStar(require("./util/appUtil"));
const events_1 = require("events");
const rpcSocketPool_1 = require("./components/rpcSocketPool");
class Application extends events_1.EventEmitter {
    constructor() {
        super();
        this.appName = "hello world"; // App name
        this.hasStarted = false; // Whether has started
        this.main = ""; // Startup file
        this.base = path.dirname(require.main.filename); // Root path
        this.routeConfig = []; // route.ts
        this.masterConfig = {}; // master.ts
        this.serversConfig = {}; // servers.ts
        this.routeConfig2 = []; // route.ts  (split)
        this.clientNum = 0; // Number of all socket connections
        this.clients = {}; // Sockets that have been binded
        this.settings = {}; // User set，get  
        this.servers = {}; // All user servers that are running
        this.serversIdMap = {}; // All user servers that are running (Dictionary format)
        this.serverInfo = {}; // The configuration of this server
        this.isDaemon = false; // Whether to run in the background
        this.env = ""; // environment
        this.serverId = ""; // Server name id, the unique identifier of the server
        this.serverType = ""; // Server type
        this.frontend = false; // Is it a front-end server
        this.startMode = "all"; // Start Mode:  all / alone
        this.startTime = 0; // Start time
        this.router = {}; // Pre-selection when routing messages to the backend
        this.rpc = null; // Rpc packaging
        this.rpcAwait = null; // Rpc await packaging
        this.rpcPool = new rpcSocketPool_1.RpcSocketPool(); // Rpc socket pool
        this.logger = function () { }; // Internal msg log output
        this.msgEncode = null;
        this.msgDecode = null;
        this.protoEncode = null;
        this.protoDecode = null;
        this.someconfig = {}; // Partially open configuration
        this.noRpcMatrix = {}; // The configuration of not establishing a socket connection between servers
        this.frontendServer = null;
        this.backendServer = null;
        appUtil.defaultConfiguration(this);
    }
    /**
     * Start up
     */
    start() {
        if (this.hasStarted) {
            console.error("the app has already started");
            return;
        }
        this.hasStarted = true;
        this.startTime = new Date().getTime();
        appUtil.startServer(this);
    }
    setConfig(key, value) {
        this.someconfig[key] = value;
        if (key === "logger") {
            this.logger = value;
        }
        else if (key === "rpc") {
            let noRpcMatrix = value["noRpcMatrix"] || {};
            for (let svrT1 in noRpcMatrix) {
                let arr = noRpcMatrix[svrT1];
                for (let svrT2 of arr) {
                    this.noRpcMatrix[appUtil.getNoRpcKey(svrT1, svrT2)] = true;
                }
            }
        }
    }
    /**
     * Set key-value pairs
     */
    set(key, value) {
        this.settings[key] = value;
        return value;
    }
    /**
     * Get the value corresponding to the key
     */
    get(key) {
        return this.settings[key];
    }
    /**
     * Delete a key-value pair
     */
    delete(key) {
        delete this.settings[key];
    }
    /**
     * Get the server array according to the server type
     */
    getServersByType(serverType) {
        return this.servers[serverType] || [];
    }
    /**
     * Get a server configuration
     */
    getServerById(serverId) {
        return this.serversIdMap[serverId];
    }
    /**
     * Routing configuration (deciding which backend to call)
     * @param serverType Back-end server type
     * @param routeFunc Configuration function
     */
    route(serverType, routeFunc) {
        this.router[serverType] = routeFunc;
    }
    /**
     * get session by uid
     */
    getSession(uid) {
        let client = this.clients[uid];
        if (client) {
            return client.session;
        }
        else {
            return null;
        }
    }
    /**
     * get all clients
     */
    getAllClients() {
        return this.clients;
    }
    /**
     * Send a message to the client
     * @param cmd   cmd
     * @param msg   message
     * @param uids  uid array [1,2]
     */
    sendMsgByUid(cmd, msg, uids) {
        if (msg === undefined) {
            msg = null;
        }
        let msgBuf = this.protoEncode(cmd, msg);
        let client;
        let i;
        for (i = 0; i < uids.length; i++) {
            client = this.clients[uids[i]];
            if (client) {
                client.send(msgBuf);
            }
        }
    }
    /**
     * Send messages to all clients
     * @param cmd cmd
     * @param msg message
     */
    sendAll(cmd, msg) {
        if (msg === undefined) {
            msg = null;
        }
        let data = this.protoEncode(cmd, msg);
        let uid;
        for (uid in this.clients) {
            this.clients[uid].send(data);
        }
    }
    /**
     * Send a message to the client
     * @param cmd   cmd
     * @param msg   message
     * @param uidsid  uidsid array
     */
    sendMsgByUidSid(cmd, msg, uidsid) {
        if (msg === undefined) {
            msg = null;
        }
        this.backendServer.sendMsgByUidSid(cmd, msg, uidsid);
    }
    /**
     * Send a message to the client
     * @param cmd   cmd
     * @param msg   message
     * @param group   { sid : uid[] }
     */
    sendMsgByGroup(cmd, msg, group) {
        if (msg === undefined) {
            msg = null;
        }
        this.backendServer.sendMsgByGroup(cmd, msg, group);
    }
    /**
     * Configure server execution function
     * @param type  Server type:  "all" or "gate|connector" like
     * @param cb    Execution function
     */
    configure(type, cb) {
        if (type === "all") {
            cb.call(this);
            return;
        }
        let ts = type.split("|");
        for (let i = 0; i < ts.length; i++) {
            if (this.serverType === ts[i].trim()) {
                cb.call(this);
                break;
            }
        }
    }
}
exports.default = Application;
